/* Sumo Bot Code Beta v 1.0 
 
 Outline:
 This sketch is the 10th iteration of code, for a project I was working on that is
 no longer under development. On your robot, this probably won't work. This code
 had issues on my robot, due to electrical noise from the motors. The noise 
 affected the line sensors, and the robot did some pretty weird things. 
 Because I'm not working on the project any more, I can't offer much help
 with your code, or answer many questions about my robot because really, I 
 never had much of a robot. It started and ended in the prototype stage
 because I ran out of time before the competition. 
 
 To Do:
 There are many things I would do differently now. One thing would be to use 
 functions for the directions. I have implemented this on other robots, and it
 works great and it is much easier to debug. I would also have PWM variables, so
 that the robot could change it's speed on the fly. 
 
 Implementation:
 My robot used two Sharp GP2D12 IR rangefinder sensors placed so that both could 
 detect a 10cm wide target or smaller. The edge of the ring was sensed by OPB704 
 reflectance sensors, with a custom breakout board. 
 
 Motor Driver:
 The motors are controlled by a Solarbotics L298 Compact Motor Driver kit. The 
 enable pins are labelled m1PWM, m2PWM, with m1 being left (from the back) and
 m2 being the right motor. FWD and REV pins go high on the inputs of the motor
 driver kit to control the direction of the motors. 
 
 |---Distance 1 (left)--------Distance 2 (right)----|
 |                     Front                        |
 |                                                  |
 |                                                  |
 |                                                  |
 |                     Back                         |
 |---Motor 1-----------------------------Motor 2----|
 
 Links:
 Project Page: http://letsmakerobots.com/node/24013
 Line Sensors: http://letsmakerobots.com/node/23116
 Motor Driver: http://www.solarbotics.com/products/k_cmd/
 
 
 Sumo Code 1.0 Beta
 LMR User: iRichiepoo
 January 3, 2011
 
 If you use this code, please credit me appropriately, with a link to
 the LMR project page. 
 
 */

// SETUP PINS

int m1FWD = 3;
int m1REV = 4;
int m1PWM = 5;
int m2FWD = 7;
int m2REV = 8;
int m2PWM = 6;
int lineLED = 12;    // Debug LED showing when a line is detected
int ledPin = 13;    // Main LED used for the 5 sec. countdown timer
int leftLED = 10;    // Debug LED showing when the left rangefinder detects an opponent. 
int rightLED = 11;    // Debug LED showing when the right rangefinder detects an opponent. 

// SETUP VARIABLES

int leftValue = 0;    // Left rangefinder
int rightValue = 0;    // Right rangefinder
int leftLine = 0;    // Left line sensor
int rightLine = 0;    // Right line sensor
const int lineThresh = 50;    // Threshold for white ring edge
const int distanceThresh = 90;    // Threshold for opponent
const int attackThresh = 150;    // Threshold for attacking 

// SETUP PIN MODES

void setup (){

  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(7, OUTPUT);
  pinMode(8, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);
  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);

  Serial.begin(9600);      // Starts serial 9600 baud

  start();    // Runs the start function, a 5 second delay.

}


void start (){    // Blinks the start LED and waits 5 seconds.

  // Stop motors
  analogWrite(m1PWM, 0);
  analogWrite(m2PWM, 0);

  // Blink LED
  digitalWrite(ledPin, HIGH);
  delay(1000);
  digitalWrite(ledPin, LOW);
  delay(1000);
  digitalWrite(ledPin, HIGH);
  delay(1000);
  digitalWrite(ledPin, LOW);
  delay(1000);
  digitalWrite(ledPin, HIGH);
  delay(1000);
  digitalWrite(ledPin, LOW);

} 


void loop (){    // Contains all of the code for line and opponent detection capabilities. 

  // Line Detection


  int leftLine = analogRead(2);

  int rightLine = analogRead(3);

  Serial.print("Left Line : ");
  Serial.println(leftLine);

  Serial.print("Right Line: ");
  Serial.println(rightLine);
  const int lineThresh = 110;
  const int distanceThresh = 90;



  if(leftLine > lineThresh || rightLine > lineThresh){

    digitalWrite(ledPin, HIGH);

    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, LOW);
    digitalWrite(m1REV, HIGH);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, LOW);  
    digitalWrite(m2REV, HIGH);

    delay(300);    // Back up 0.3 seconds

    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, LOW);
    digitalWrite(m1REV, HIGH);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, HIGH);  
    digitalWrite(m2REV, LOW);

    delay(700);   // Turn Left 0.7 seconds

    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, HIGH);
    digitalWrite(m1REV, LOW);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, HIGH);  
    digitalWrite(m2REV, LOW);    // Forward

    digitalWrite(ledPin, LOW);
  }

  else{
    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, HIGH);
    digitalWrite(m1REV, LOW);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, HIGH);  
    digitalWrite(m2REV, LOW);    // Forward
  }



  // Opponent Detection Code

  int leftValue = analogRead(0);
  int rightValue = analogRead(1);


  // Left Sensor Sees Opponent

  if(leftValue > rightValue + distanceThresh){
    analogWrite(m1PWM, 0);
    digitalWrite(m1FWD, LOW);
    digitalWrite(m1REV, HIGH);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, HIGH);  
    digitalWrite(m2REV, LOW);    

  }

  // Right Sensor Sees Opponent

  else if(rightValue > leftValue + distanceThresh){
    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, HIGH);
    digitalWrite(m1REV, LOW);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, LOW);  
    digitalWrite(m2REV, HIGH);    
  }

  // Both Sensors See Opponent

  else if(leftValue > attackThresh && rightValue > attackThresh){
    digitalWrite(ledPin, HIGH);
    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, HIGH);
    digitalWrite(m1REV, LOW);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, HIGH);  
    digitalWrite(m2REV, LOW); 
    digitalWrite(ledPin, LOW);
  }

  else if(leftValue + 30 <= rightValue || leftValue - 30 >= rightValue){
    digitalWrite(leftLED, HIGH);
    digitalWrite(rightLED, HIGH);

    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, HIGH);
    digitalWrite(m1REV, LOW);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, HIGH);  
    digitalWrite(m2REV, LOW); 

    digitalWrite(leftLED, LOW);
    digitalWrite(rightLED, LOW);
  }



  // Seek Opponent

  else{
    analogWrite(m1PWM, 255);
    digitalWrite(m1FWD, HIGH);
    digitalWrite(m1REV, LOW);

    analogWrite(m2PWM, 255);
    digitalWrite(m2FWD, LOW);  
    digitalWrite(m2REV, HIGH); 
  }

}    // End loop. Goodbye. 


















